@@ -40,7 +40,7 @@ window.condition_popup = new class {
 	}
 
 	_loadViews() {
-		if (this.form.querySelector('#condition-type').value == <?= CONDITION_TYPE_SERVICE ?>) {
+		if (this.form.querySelector('#condition-type').value == <?= ZBX_CONDITION_TYPE_SERVICE ?>) {
 			$('#service-new-condition')
 				.multiSelect('getSelectButton')
 				.addEventListener('click', () => this.selectServices());
@@ -148,7 +148,9 @@ window.condition_popup = new class {
 	}
 
 	selectServices() {
-		const overlay = PopUp('popup.services', {title: t('Services')}, {dialogueid: 'services'});
+		const overlay = PopUp('popup.services', {title: t('Services')},
+			{dialogueid: 'services', dialogue_class: 'modal-popup-generic'}
+		);
 		overlay.$dialogue[0].addEventListener('dialogue.submit', (e) => {
 			const data = [];
 			for (const service of e.detail) {
