@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,9 +26,7 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of item prototypes');
 $page['file'] = 'disc_prototypes.php';
-$page['scripts'] = ['effects.js', 'class.cviewswitcher.js', 'multilineinput.js', 'multiselect.js', 'items.js',
-	'textareaflexible.js', 'class.tab-indicators.js'
-];
+$page['scripts'] = ['effects.js', 'multilineinput.js', 'items.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -61,7 +59,7 @@ $fields = [
 													' && isset({key}) && strncmp({key}, "mqtt.get", 8) === 0)',
 										_('Update interval')
 									],
-	'delay_flex' =>					[T_ZBX_STR, O_OPT, null,	null,			null],
+	'delay_flex' =>					[T_ZBX_STR, O_OPT, P_ONLY_TD_ARRAY,	null,	null],
 	'status' =>						[T_ZBX_INT, O_OPT, null,	IN([ITEM_STATUS_ACTIVE, ITEM_STATUS_DISABLED]), null],
 	'discover' =>					[T_ZBX_INT, O_OPT, null,	IN([ZBX_PROTOTYPE_DISCOVER, ZBX_PROTOTYPE_NO_DISCOVER]), null],
 	'type' =>						[T_ZBX_INT, O_OPT, null,
@@ -124,8 +122,8 @@ $fields = [
 										'(isset({add}) || isset({update})) && isset({value_type})'.
 											' && {value_type} == '.ITEM_VALUE_TYPE_LOG
 									],
-	'preprocessing' =>				[T_ZBX_STR, O_OPT, P_NO_TRIM,	null,	null],
-	'group_itemid' =>				[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
+	'preprocessing' =>				[null,      O_OPT, P_NO_TRIM|P_ONLY_TD_ARRAY,	null,	null],
+	'group_itemid' =>				[T_ZBX_INT, O_OPT, P_ONLY_ARRAY,				DB_ID,	null],
 	'history_mode' =>				[T_ZBX_INT, O_OPT, null,	IN([ITEM_STORAGE_OFF, ITEM_STORAGE_CUSTOM]), null],
 	'history' =>					[T_ZBX_STR, O_OPT, null,	null,
 										'(isset({add}) || isset({update}))'.
@@ -153,10 +151,10 @@ $fields = [
 											' && {type} == '.ITEM_TYPE_HTTPAGENT,
 										_('URL')
 									],
-	'query_fields' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
-	'parameters' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
-	'posts' =>						[T_ZBX_STR, O_OPT, null,	null,		null],
-	'status_codes' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
+	'query_fields' =>				[T_ZBX_STR, O_OPT, P_ONLY_TD_ARRAY,	null,	null],
+	'parameters' =>					[T_ZBX_STR, O_OPT, P_ONLY_TD_ARRAY,	null,	null],
+	'posts' =>						[T_ZBX_STR, O_OPT, null,			null,	null],
+	'status_codes' =>				[T_ZBX_STR, O_OPT, null,			null,	null],
 	'follow_redirects' =>			[T_ZBX_INT, O_OPT, null,
 										IN([HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF, HTTPTEST_STEP_FOLLOW_REDIRECTS_ON]),
 										null
@@ -165,8 +163,8 @@ $fields = [
 										IN([ZBX_POSTTYPE_RAW, ZBX_POSTTYPE_JSON, ZBX_POSTTYPE_XML]),
 										null
 									],
-	'http_proxy' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
-	'headers' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
+	'http_proxy' =>					[T_ZBX_STR, O_OPT, null,			null,	null],
+	'headers' =>					[T_ZBX_STR, O_OPT, P_ONLY_TD_ARRAY,	null,	null],
 	'retrieve_mode' =>				[T_ZBX_INT, O_OPT, null,
 										IN([HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS,
 											HTTPTEST_STEP_RETRIEVE_MODE_BOTH
@@ -188,40 +186,40 @@ $fields = [
 	'ssl_key_file' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
 	'ssl_key_password' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
 	'verify_peer' =>				[T_ZBX_INT, O_OPT, null,
-										IN([HTTPTEST_VERIFY_PEER_OFF, HTTPTEST_VERIFY_PEER_ON]),
+										IN([ZBX_HTTP_VERIFY_PEER_OFF, ZBX_HTTP_VERIFY_PEER_ON]),
 										null
 									],
 	'verify_host' =>				[T_ZBX_INT, O_OPT, null,
-										IN([HTTPTEST_VERIFY_HOST_OFF, HTTPTEST_VERIFY_HOST_ON]),
+										IN([ZBX_HTTP_VERIFY_HOST_OFF, ZBX_HTTP_VERIFY_HOST_ON]),
 										null
 									],
 	'http_authtype' =>				[T_ZBX_INT, O_OPT, null,
-										IN([HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM,
-											HTTPTEST_AUTH_KERBEROS, HTTPTEST_AUTH_DIGEST
+										IN([ZBX_HTTP_AUTH_NONE, ZBX_HTTP_AUTH_BASIC, ZBX_HTTP_AUTH_NTLM,
+											ZBX_HTTP_AUTH_KERBEROS, ZBX_HTTP_AUTH_DIGEST
 										]),
 										null
 									],
 	'http_username' =>				[T_ZBX_STR, O_OPT, null,	null,
 										'(isset({add}) || isset({update})) && isset({http_authtype})'.
-											' && ({http_authtype} == '.HTTPTEST_AUTH_BASIC.
-												' || {http_authtype} == '.HTTPTEST_AUTH_NTLM.
-												' || {http_authtype} == '.HTTPTEST_AUTH_KERBEROS.
-												' || {http_authtype} == '.HTTPTEST_AUTH_DIGEST.
+											' && ({http_authtype} == '.ZBX_HTTP_AUTH_BASIC.
+												' || {http_authtype} == '.ZBX_HTTP_AUTH_NTLM.
+												' || {http_authtype} == '.ZBX_HTTP_AUTH_KERBEROS.
+												' || {http_authtype} == '.ZBX_HTTP_AUTH_DIGEST.
 											')',
 										_('Username')
 									],
 	'http_password' =>				[T_ZBX_STR, O_OPT, null,	null,
 										'(isset({add}) || isset({update})) && isset({http_authtype})'.
-											' && ({http_authtype} == '.HTTPTEST_AUTH_BASIC.
-												' || {http_authtype} == '.HTTPTEST_AUTH_NTLM.
-												' || {http_authtype} == '.HTTPTEST_AUTH_KERBEROS.
-												' || {http_authtype} == '.HTTPTEST_AUTH_DIGEST.
+											' && ({http_authtype} == '.ZBX_HTTP_AUTH_BASIC.
+												' || {http_authtype} == '.ZBX_HTTP_AUTH_NTLM.
+												' || {http_authtype} == '.ZBX_HTTP_AUTH_KERBEROS.
+												' || {http_authtype} == '.ZBX_HTTP_AUTH_DIGEST.
 											')',
 										_('Password')
 									],
-	'visible' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
+	'visible' =>					[T_ZBX_STR, O_OPT, P_ONLY_ARRAY,	null,	null],
 	'context' =>					[T_ZBX_STR, O_MAND, P_SYS,	IN('"host", "template"'),	null],
-	'tags' =>						[T_ZBX_STR, O_OPT, null,	null,		null],
+	'tags' =>						[T_ZBX_STR, O_OPT, P_ONLY_TD_ARRAY,	null,	null],
 	'show_inherited_tags' =>		[T_ZBX_INT, O_OPT, null,	IN([0,1]),	null],
 	// actions
 	'action' =>						[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
@@ -237,7 +235,7 @@ $fields = [
 	'delete' =>						[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'cancel' =>						[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'form' =>						[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
-	'form_refresh' =>				[T_ZBX_INT, O_OPT, null,	null,		null],
+	'form_refresh' =>				[T_ZBX_INT, O_OPT, P_SYS,	null,		null],
 	// filter
 	'filter_set' =>					[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	// sort and sortorder
@@ -246,58 +244,38 @@ $fields = [
 									],
 	'sortorder' =>					[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
 ];
+
 $valid_input = check_fields($fields);
 
 $_REQUEST['params'] = getRequest($paramsFieldName, '');
 unset($_REQUEST[$paramsFieldName]);
 
-// permissions
-$discoveryRule = API::DiscoveryRule()->get([
-	'output' => ['hostid'],
+// Permissions.
+$lld_rules = API::DiscoveryRule()->get([
+	'output' => ['itemid', 'hostid'],
+	'selectHosts' => ['status'],
 	'itemids' => getRequest('parent_discoveryid'),
 	'editable' => true
 ]);
-$discoveryRule = reset($discoveryRule);
-if (!$discoveryRule) {
+
+if (!$lld_rules) {
 	access_deny();
 }
 
-$itemPrototypeId = getRequest('itemid');
-if ($itemPrototypeId) {
-	$item_prorotypes = API::ItemPrototype()->get([
+$itemid = getRequest('itemid');
+
+if ($itemid) {
+	$item_prototypes = API::ItemPrototype()->get([
 		'output' => [],
-		'itemids' => $itemPrototypeId,
+		'itemids' => $itemid,
 		'editable' => true
 	]);
 
-	if (!$item_prorotypes) {
+	if (!$item_prototypes) {
 		access_deny();
 	}
 }
 
-// Convert CR+LF to LF in preprocessing script.
-if (hasRequest('preprocessing')) {
-	foreach ($_REQUEST['preprocessing'] as &$step) {
-		if ($step['type'] == ZBX_PREPROC_SCRIPT) {
-			$step['params'][0] = CRLFtoLF($step['params'][0]);
-		}
-	}
-	unset($step);
-}
-
-$tags = getRequest('tags', []);
-foreach ($tags as $key => $tag) {
-	if ($tag['tag'] === '' && $tag['value'] === '') {
-		unset($tags[$key]);
-	}
-	elseif (array_key_exists('type', $tag) && !($tag['type'] & ZBX_PROPERTY_OWN)) {
-		unset($tags[$key]);
-	}
-	else {
-		unset($tags[$key]['type']);
-	}
-}
-
 /*
  * Actions
  */
@@ -314,297 +292,149 @@ if (hasRequest('delete') && hasRequest('itemid')) {
 	unset($_REQUEST['itemid'], $_REQUEST['form']);
 }
 elseif (hasRequest('add') || hasRequest('update')) {
-	$result = true;
-	DBstart();
+	try {
+		$type = (int) getRequest('type', DB::getDefault('items', 'type'));
+		$key = getRequest('key', DB::getDefault('items', 'key_'));
 
-	$delay = getRequest('delay', DB::getDefault('items', 'delay'));
-	$type = getRequest('type', ITEM_TYPE_ZABBIX);
-
-	/*
-	 * "delay_flex" is a temporary field that collects flexible and scheduling intervals separated by a semicolon.
-	 * In the end, custom intervals together with "delay" are stored in the "delay" variable.
-	 */
-	if ($type != ITEM_TYPE_TRAPPER && $type != ITEM_TYPE_SNMPTRAP
-			&& ($type != ITEM_TYPE_ZABBIX_ACTIVE || strncmp(getRequest('key'), 'mqtt.get', 8) !== 0)
-			&& hasRequest('delay_flex')) {
-		$intervals = [];
-		$simple_interval_parser = new CSimpleIntervalParser([
-			'usermacros' => true,
-			'lldmacros' => true
-		]);
-		$time_period_parser = new CTimePeriodParser([
-			'usermacros' => true,
-			'lldmacros' => true
-		]);
-		$scheduling_interval_parser = new CSchedulingIntervalParser([
-			'usermacros' => true,
-			'lldmacros' => true
-		]);
-
-		foreach (getRequest('delay_flex') as $interval) {
-			if ($interval['type'] == ITEM_DELAY_FLEXIBLE) {
-				if ($interval['delay'] === '' && $interval['period'] === '') {
-					continue;
-				}
-
-				if ($simple_interval_parser->parse($interval['delay']) != CParser::PARSE_SUCCESS) {
-					$result = false;
-					info(_s('Invalid interval "%1$s".', $interval['delay']));
-					break;
-				}
-				elseif ($time_period_parser->parse($interval['period']) != CParser::PARSE_SUCCESS) {
-					$result = false;
-					info(_s('Invalid interval "%1$s".', $interval['period']));
-					break;
-				}
-
-				$intervals[] = $interval['delay'].'/'.$interval['period'];
-			}
-			else {
-				if ($interval['schedule'] === '') {
-					continue;
-				}
-
-				if ($scheduling_interval_parser->parse($interval['schedule']) != CParser::PARSE_SUCCESS) {
-					$result = false;
-					info(_s('Invalid interval "%1$s".', $interval['schedule']));
-					break;
-				}
-
-				$intervals[] = $interval['schedule'];
-			}
+		if (isItemExampleKey($type, $key)) {
+			throw new Exception();
 		}
 
-		if ($intervals) {
-			$delay .= ';'.implode(';', $intervals);
+		$delay_flex = getRequest('delay_flex', []);
+
+		if (!isValidCustomIntervals($delay_flex, true)) {
+			throw new Exception();
 		}
-	}
 
-	if ($result) {
-		$preprocessing = getRequest('preprocessing', []);
+		$value_type = (int) getRequest('value_type', DB::getDefault('items', 'value_type'));
+		$trends_default = in_array($value_type, [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64])
+			? DB::getDefault('items', 'trends')
+			: 0;
 
-		foreach ($preprocessing as &$step) {
-			switch ($step['type']) {
-				case ZBX_PREPROC_MULTIPLIER:
-				case ZBX_PREPROC_PROMETHEUS_TO_JSON:
-					$step['params'] = trim($step['params'][0]);
-					break;
+		$request_method = getRequest('request_method', DB::getDefault('items', 'request_method'));
+		$retrieve_mode_default = $request_method == HTTPCHECK_REQUEST_HEAD
+			? HTTPTEST_STEP_RETRIEVE_MODE_HEADERS
+			: DB::getDefault('items', 'retrieve_mode');
 
-				case ZBX_PREPROC_RTRIM:
-				case ZBX_PREPROC_LTRIM:
-				case ZBX_PREPROC_TRIM:
-				case ZBX_PREPROC_XPATH:
-				case ZBX_PREPROC_JSONPATH:
-				case ZBX_PREPROC_VALIDATE_REGEX:
-				case ZBX_PREPROC_VALIDATE_NOT_REGEX:
-				case ZBX_PREPROC_ERROR_FIELD_JSON:
-				case ZBX_PREPROC_ERROR_FIELD_XML:
-				case ZBX_PREPROC_THROTTLE_TIMED_VALUE:
-				case ZBX_PREPROC_SCRIPT:
-					$step['params'] = $step['params'][0];
-					break;
-
-				case ZBX_PREPROC_VALIDATE_RANGE:
-				case ZBX_PREPROC_PROMETHEUS_PATTERN:
-					foreach ($step['params'] as &$param) {
-						$param = trim($param);
-					}
-					unset($param);
-
-					$step['params'] = implode("\n", $step['params']);
-					break;
-
-				case ZBX_PREPROC_REGSUB:
-				case ZBX_PREPROC_ERROR_FIELD_REGEX:
-				case ZBX_PREPROC_STR_REPLACE:
-					$step['params'] = implode("\n", $step['params']);
-					break;
-
-				// ZBX-16642
-				case ZBX_PREPROC_CSV_TO_JSON:
-					if (!array_key_exists(2, $step['params'])) {
-						$step['params'][2] = ZBX_PREPROC_CSV_NO_HEADER;
-					}
-					$step['params'] = implode("\n", $step['params']);
-					break;
-
-				default:
-					$step['params'] = '';
-			}
-
-			$step += [
-				'error_handler' => ZBX_PREPROC_FAIL_DEFAULT,
-				'error_handler_params' => ''
-			];
-		}
-		unset($step);
-
-		$item = [
-			'name'			=> getRequest('name'),
-			'description'	=> getRequest('description'),
-			'key_'			=> getRequest('key'),
-			'hostid'		=> $discoveryRule['hostid'],
-			'interfaceid'	=> getRequest('interfaceid'),
-			'delay'			=> $delay,
-			'status'		=> getRequest('status', ITEM_STATUS_DISABLED),
-			'discover'		=> getRequest('discover', ZBX_PROTOTYPE_DISCOVER),
-			'type'			=> getRequest('type'),
-			'snmp_oid'		=> getRequest('snmp_oid'),
-			'value_type'	=> getRequest('value_type'),
-			'trapper_hosts'	=> getRequest('trapper_hosts'),
-			'history'		=> (getRequest('history_mode', ITEM_STORAGE_CUSTOM) == ITEM_STORAGE_OFF)
+		$input = [
+			'name' => getRequest('name', DB::getDefault('items', 'name')),
+			'type' => $type,
+			'key_' => $key,
+			'value_type' => $value_type,
+			'units' => getRequest('units', DB::getDefault('items', 'units')),
+			'history' => getRequest('history_mode', ITEM_STORAGE_CUSTOM) == ITEM_STORAGE_OFF
 				? ITEM_NO_STORAGE_VALUE
-				: getRequest('history'),
-			'units'			=> getRequest('units'),
-			'logtimefmt'	=> getRequest('logtimefmt'),
-			'valuemapid'	=> getRequest('valuemapid', 0),
-			'authtype'		=> getRequest('authtype'),
-			'username'		=> getRequest('username'),
-			'password'		=> getRequest('password'),
-			'publickey'		=> getRequest('publickey'),
-			'privatekey'	=> getRequest('privatekey'),
-			'params'		=> getRequest('params'),
-			'ipmi_sensor'	=> getRequest('ipmi_sensor'),
-			'ruleid'		=> getRequest('parent_discoveryid')
+				: getRequest('history', DB::getDefault('items', 'history')),
+			'trends' => getRequest('trends_mode', ITEM_STORAGE_CUSTOM) == ITEM_STORAGE_OFF
+				? ITEM_NO_STORAGE_VALUE
+				: getRequest('trends', $trends_default),
+			'valuemapid' => getRequest('valuemapid', 0),
+			'inventory_link' => getRequest('inventory_link', DB::getDefault('items', 'inventory_link')),
+			'logtimefmt' => getRequest('logtimefmt', DB::getDefault('items', 'logtimefmt')),
+			'description' => getRequest('description', DB::getDefault('items', 'description')),
+			'status' => getRequest('status', ITEM_STATUS_DISABLED),
+			'discover' => getRequest('discover', DB::getDefault('items', 'discover')),
+			'tags' => prepareItemTags(getRequest('tags', [])),
+			'preprocessing' => normalizeItemPreprocessingSteps(getRequest('preprocessing', [])),
+
+			// Type fields.
+			// The fields used for multiple item types.
+			'interfaceid' => getRequest('interfaceid', 0),
+			'authtype' => $type == ITEM_TYPE_HTTPAGENT
+				? getRequest('http_authtype', DB::getDefault('items', 'authtype'))
+				: getRequest('authtype', DB::getDefault('items', 'authtype')),
+			'username' => $type == ITEM_TYPE_HTTPAGENT
+				? getRequest('http_username', DB::getDefault('items', 'username'))
+				: getRequest('username', DB::getDefault('items', 'username')),
+			'password' => $type == ITEM_TYPE_HTTPAGENT
+				? getRequest('http_password', DB::getDefault('items', 'password'))
+				: getRequest('password', DB::getDefault('items', 'password')),
+			'params' => getRequest('params', DB::getDefault('items', 'params')),
+			'timeout' => getRequest('timeout', DB::getDefault('items', 'timeout')),
+			'delay' => getDelayWithCustomIntervals(getRequest('delay', DB::getDefault('items', 'delay')), $delay_flex),
+			'trapper_hosts' => getRequest('trapper_hosts', DB::getDefault('items', 'trapper_hosts')),
+
+			// Dependent item type specific fields.
+			'master_itemid' => getRequest('master_itemid', 0),
+
+			// HTTP Agent item type specific fields.
+			'url' => getRequest('url', DB::getDefault('items', 'url')),
+			'query_fields' => prepareItemQueryFields(getRequest('query_fields', [])),
+			'request_method' => $request_method,
+			'post_type' => getRequest('post_type', DB::getDefault('items', 'post_type')),
+			'posts' => getRequest('posts', DB::getDefault('items', 'posts')),
+			'headers' => prepareItemHeaders(getRequest('headers', [])),
+			'status_codes' => getRequest('status_codes', DB::getDefault('items', 'status_codes')),
+			'follow_redirects' => getRequest('follow_redirects', HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF),
+			'retrieve_mode' => getRequest('retrieve_mode', $retrieve_mode_default),
+			'output_format' => getRequest('output_format', DB::getDefault('items', 'output_format')),
+			'http_proxy' => getRequest('http_proxy', DB::getDefault('items', 'http_proxy')),
+			'verify_peer' => getRequest('verify_peer', DB::getDefault('items', 'verify_peer')),
+			'verify_host' => getRequest('verify_host', DB::getDefault('items', 'verify_host')),
+			'ssl_cert_file' => getRequest('ssl_cert_file', DB::getDefault('items', 'ssl_cert_file')),
+			'ssl_key_file' => getRequest('ssl_key_file', DB::getDefault('items', 'ssl_key_file')),
+			'ssl_key_password' => getRequest('ssl_key_password', DB::getDefault('items', 'ssl_key_password')),
+			'allow_traps' => getRequest('allow_traps', DB::getDefault('items', 'allow_traps')),
+
+			// IPMI item type specific fields.
+			'ipmi_sensor' => getRequest('ipmi_sensor', DB::getDefault('items', 'ipmi_sensor')),
+
+			// JMX item type specific fields.
+			'jmx_endpoint' => getRequest('jmx_endpoint', DB::getDefault('items', 'jmx_endpoint')),
+
+			// Script item type specific fields.
+			'parameters' => prepareItemParameters(getRequest('parameters', [])),
+
+			// SNMP item type specific fields.
+			'snmp_oid' => getRequest('snmp_oid', DB::getDefault('items', 'snmp_oid')),
+
+			// SSH item type specific fields.
+			'publickey' => getRequest('publickey', DB::getDefault('items', 'publickey')),
+			'privatekey' => getRequest('privatekey', DB::getDefault('items', 'privatekey'))
 		];
 
-		if ($item['type'] == ITEM_TYPE_SCRIPT) {
-			$script_item = [
-				'parameters' => getRequest('parameters', []),
-				'timeout' => getRequest('timeout', DB::getDefault('items', 'timeout'))
+		$result = true;
+
+		if (hasRequest('add')) {
+			$item = [
+				'hostid' => $lld_rules[0]['hostid'],
+				'ruleid' => $lld_rules[0]['itemid']
 			];
 
-			$item = prepareScriptItemFormData($script_item) + $item;
-		}
+			$item += getSanitizedItemFields($input + [
+				'templateid' => '0',
+				'flags' => ZBX_FLAG_DISCOVERY_PROTOTYPE,
+				'hosts' => $lld_rules[0]['hosts']
+			]);
 
-		if ($item['type'] == ITEM_TYPE_JMX) {
-			$item['jmx_endpoint'] = getRequest('jmx_endpoint', '');
-		}
+			$response = API::ItemPrototype()->create($item);
 
-		if ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT || $item['value_type'] == ITEM_VALUE_TYPE_UINT64) {
-			$item['trends'] = (getRequest('trends_mode', ITEM_STORAGE_CUSTOM) == ITEM_STORAGE_OFF)
-				? ITEM_NO_STORAGE_VALUE
-				: getRequest('trends');
-		}
-
-		if ($item['type'] == ITEM_TYPE_DEPENDENT) {
-			$item['master_itemid'] = getRequest('master_itemid');
+			if ($response === false) {
+				throw new Exception();
+			}
 		}
 
 		if (hasRequest('update')) {
-			$itemId = getRequest('itemid');
-
-			$db_item = API::ItemPrototype()->get([
-				'output' => ['type', 'snmp_oid', 'hostid', 'name', 'key_', 'delay', 'history',
-					'trends', 'status', 'value_type', 'trapper_hosts', 'units',
-					'logtimefmt', 'templateid', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
-					'password', 'publickey', 'privatekey', 'interfaceid', 'description', 'jmx_endpoint',
-					'master_itemid', 'timeout', 'url', 'query_fields', 'posts', 'status_codes', 'follow_redirects',
-					'post_type', 'http_proxy', 'headers', 'retrieve_mode', 'request_method', 'output_format',
-					'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host', 'allow_traps',
-					'discover', 'parameters'
-				],
-				'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
-				'selectTags' => ['tag', 'value'],
-				'itemids' => [$itemId]
+			$db_items = API::ItemPrototype()->get([
+				'output' => ['templateid', 'type', 'key_', 'value_type', 'authtype', 'allow_traps'],
+				'itemids' => $itemid
 			]);
 
-			$db_item = $db_item[0];
+			$item = getSanitizedItemFields($input + $db_items[0] + [
+				'flags' => ZBX_FLAG_DISCOVERY_PROTOTYPE,
+				'hosts' => $lld_rules[0]['hosts']
+			]);
 
-			if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
-				$http_item = [
-					'timeout' => getRequest('timeout', DB::getDefault('items', 'timeout')),
-					'url' => getRequest('url'),
-					'query_fields' => getRequest('query_fields', []),
-					'posts' => getRequest('posts'),
-					'status_codes' => getRequest('status_codes', DB::getDefault('items', 'status_codes')),
-					'follow_redirects' => (int) getRequest('follow_redirects'),
-					'post_type' => (int) getRequest('post_type'),
-					'http_proxy' => getRequest('http_proxy'),
-					'headers' => getRequest('headers', []),
-					'retrieve_mode' => (int) getRequest('retrieve_mode'),
-					'request_method' => (int) getRequest('request_method'),
-					'output_format' => (int) getRequest('output_format'),
-					'allow_traps' => (int) getRequest('allow_traps', HTTPCHECK_ALLOW_TRAPS_OFF),
-					'ssl_cert_file' => getRequest('ssl_cert_file'),
-					'ssl_key_file' => getRequest('ssl_key_file'),
-					'ssl_key_password' => getRequest('ssl_key_password'),
-					'verify_peer' => (int) getRequest('verify_peer'),
-					'verify_host' => (int) getRequest('verify_host'),
-					'authtype' => getRequest('http_authtype', HTTPTEST_AUTH_NONE),
-					'username' => getRequest('http_username', ''),
-					'password' => getRequest('http_password', '')
-				];
-				$item = prepareItemHttpAgentFormData($http_item) + $item;
+			$response = API::ItemPrototype()->update(['itemid' => $itemid] + $item);
+
+			if ($response === false) {
+				throw new Exception();
 			}
-
-			if ($db_item['type'] == $item['type']) {
-				$item = CArrayHelper::unsetEqualValues($item, $db_item);
-			}
-
-			$item['itemid'] = $itemId;
-
-			if ($db_item['preprocessing'] !== $preprocessing) {
-				$item['preprocessing'] = $preprocessing;
-			}
-
-			$compare = function($arr, $arr2) {
-				return (array_combine(array_column($arr, 'name'), array_column($arr, 'value')) ==
-					array_combine(array_column($arr2, 'name'), array_column($arr2, 'value'))
-				);
-			};
-			if (getRequest('type') == ITEM_TYPE_SCRIPT && $db_item['type'] == getRequest('type')
-					&& $compare($db_item['parameters'], $item['parameters'])) {
-				unset($item['parameters']);
-			}
-
-			CArrayHelper::sort($db_item['tags'], ['tag', 'value']);
-			CArrayHelper::sort($tags, ['tag', 'value']);
-			if (array_values($db_item['tags']) !== array_values($tags)) {
-				$item['tags'] = $tags;
-			}
-
-			$result = API::ItemPrototype()->update($item);
-		}
-		else {
-			if (getRequest('type') == ITEM_TYPE_HTTPAGENT) {
-				$http_item = [
-					'timeout' => getRequest('timeout', DB::getDefault('items', 'timeout')),
-					'url' => getRequest('url'),
-					'query_fields' => getRequest('query_fields', []),
-					'posts' => getRequest('posts'),
-					'status_codes' => getRequest('status_codes', DB::getDefault('items', 'status_codes')),
-					'follow_redirects' => (int) getRequest('follow_redirects'),
-					'post_type' => (int) getRequest('post_type'),
-					'http_proxy' => getRequest('http_proxy'),
-					'headers' => getRequest('headers', []),
-					'retrieve_mode' => (int) getRequest('retrieve_mode'),
-					'request_method' => (int) getRequest('request_method'),
-					'output_format' => (int) getRequest('output_format'),
-					'allow_traps' => (int) getRequest('allow_traps', HTTPCHECK_ALLOW_TRAPS_OFF),
-					'ssl_cert_file' => getRequest('ssl_cert_file'),
-					'ssl_key_file' => getRequest('ssl_key_file'),
-					'ssl_key_password' => getRequest('ssl_key_password'),
-					'verify_peer' => (int) getRequest('verify_peer'),
-					'verify_host' => (int) getRequest('verify_host'),
-					'authtype' => getRequest('http_authtype', HTTPTEST_AUTH_NONE),
-					'username' => getRequest('http_username', ''),
-					'password' => getRequest('http_password', '')
-				];
-				$item = prepareItemHttpAgentFormData($http_item) + $item;
-			}
-
-			if ($preprocessing) {
-				$item['preprocessing'] = $preprocessing;
-			}
-
-			$item['tags'] = $tags;
-
-			$result = API::ItemPrototype()->create($item);
 		}
 	}
-
-	$result = DBend($result);
+	catch (Exception $e) {
+		$result = false;
+	}
 
 	if (hasRequest('add')) {
 		show_messages($result, _('Item prototype added'), _('Cannot add item prototype'));
@@ -701,6 +531,7 @@ if (hasRequest('form') || (hasRequest('clone') && getRequest('itemid') != 0)) {
 		]);
 		$itemPrototype = reset($itemPrototype);
 
+		$i = 0;
 		foreach ($itemPrototype['preprocessing'] as &$step) {
 			if ($step['type'] == ZBX_PREPROC_SCRIPT) {
 				$step['params'] = [$step['params'], ''];
@@ -708,6 +539,7 @@ if (hasRequest('form') || (hasRequest('clone') && getRequest('itemid') != 0)) {
 			else {
 				$step['params'] = explode("\n", $step['params']);
 			}
+			$step['sortorder'] = $i++;
 		}
 		unset($step);
 
@@ -754,6 +586,7 @@ if (hasRequest('form') || (hasRequest('clone') && getRequest('itemid') != 0)) {
 
 	$form_action = (hasRequest('clone') && getRequest('itemid') != 0) ? 'clone' : getRequest('form');
 	$data = getItemFormData($itemPrototype, ['form' => $form_action]);
+	CArrayHelper::sort($data['preprocessing'], ['sortorder']);
 	$data['preprocessing_test_type'] = CControllerPopupItemTestEdit::ZBX_TEST_TYPE_ITEM_PROTOTYPE;
 	$data['preprocessing_types'] = CItemPrototype::SUPPORTED_PREPROCESSING_TYPES;
 	$data['trends_default'] = DB::getDefault('items', 'trends');
@@ -801,9 +634,10 @@ else {
 	CProfile::update($prefix.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
 
 	$data = [
+		'form_refresh' => getRequest('form_refresh', 0),
 		'form' => getRequest('form'),
 		'parent_discoveryid' => getRequest('parent_discoveryid'),
-		'hostid' => $discoveryRule['hostid'],
+		'hostid' => $lld_rules[0]['hostid'],
 		'sort' => $sortField,
 		'sortorder' => $sortOrder,
 		'context' => getRequest('context')
