@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -40,7 +40,6 @@ function DBconnect(&$error) {
 	$DB['TRANSACTION_NO_FAILED_SQLS'] = true; // true - if no statements failed in transaction, false - there are failed statements
 	$DB['SELECT_COUNT'] = 0; // stats
 	$DB['EXECUTE_COUNT'] = 0;
-	$DB['DB'] = null;
 
 	if (!isset($DB['TYPE'])) {
 		$error = 'Unknown database type.';
@@ -74,6 +73,7 @@ function DBconnect(&$error) {
 
 	if ($db->getError() || ($DB['ENCRYPTION'] && !$db->isConnectionSecure()) || !$db->checkDbVersion()
 			|| !$db->checkConfig()) {
+
 		$error = $db->getError();
 		return false;
 	}
@@ -109,7 +109,8 @@ function DBstart() {
 	$result = false;
 
 	if ($DB['TRANSACTIONS'] != 0) {
-		info('POSSIBLE ERROR: Used incorrect logic in database processing, started subtransaction!');
+		error('POSSIBLE ERROR: Used incorrect logic in database processing, started subtransaction!');
+
 		return $result;
 	}
 
@@ -144,15 +145,14 @@ function DBstart() {
 function DBend($doCommit = true) {
 	global $DB;
 
-	$result = false;
-
 	if (!isset($DB['DB']) || empty($DB['DB'])) {
-		return $result;
+		return false;
 	}
 
 	if ($DB['TRANSACTIONS'] == 0) {
-		info('POSSIBLE ERROR: Used incorrect logic in database processing, transaction not started!');
-		return $result;
+		error('POSSIBLE ERROR: Used incorrect logic in database processing, transaction not started!');
+
+		return false;
 	}
 
 	$DBresult = $doCommit && $DB['TRANSACTION_NO_FAILED_SQLS'];
@@ -218,50 +218,64 @@ function DBrollback() {
  * @param int $limit    max number of record to return
  * @param int $offset   return starting from $offset record
  *
- * @return resource or object, False if failed
+ * @return resource|false
  */
 function DBselect($query, $limit = null, $offset = 0) {
 	global $DB;
 
-	$result = false;
-
-	if (!isset($DB['DB']) || empty($DB['DB'])) {
-		return $result;
+	if (!array_key_exists('DB', $DB) || $DB['DB'] === null) {
+		return false;
 	}
 
-	// add the LIMIT clause
-	if(!$query = DBaddLimit($query, $limit, $offset)) {
+	$query = DBaddLimit($query, $limit, $offset);
+
+	if ($query === false) {
 		return false;
 	}
 
 	$time_start = microtime(true);
+
 	$DB['SELECT_COUNT']++;
 
+	$result = false;
+
 	switch ($DB['TYPE']) {
 		case ZBX_DB_MYSQL:
-			if (!$result = mysqli_query($DB['DB'], $query)) {
-				error('Error in query ['.$query.'] ['.mysqli_error($DB['DB']).']', 'sql');
+			try {
+				$result = mysqli_query($DB['DB'], $query);
 			}
+			catch (mysqli_sql_exception $e) {
+				trigger_error('Error in query ['.$query.'] ['.$e->getMessage().']', E_USER_WARNING);
+			}
+
 			break;
+
 		case ZBX_DB_POSTGRESQL:
 			if (!$result = pg_query($DB['DB'], $query)) {
-				error('Error in query ['.$query.'] ['.pg_last_error().']', 'sql');
+				error('Error in query ['.$query.'] ['.pg_last_error($DB['DB']).']', true);
 			}
+
 			break;
+
 		case ZBX_DB_ORACLE:
-			if (!$result = oci_parse($DB['DB'], $query)) {
-				$e = @oci_error();
-				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', 'sql');
+			$result = oci_parse($DB['DB'], $query);
+
+			if ($result === false) {
+				$e = oci_error();
+				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', true);
+
+				break;
 			}
-			elseif (!@oci_execute($result, ($DB['TRANSACTIONS'] ? OCI_DEFAULT : OCI_COMMIT_ON_SUCCESS))) {
+
+			if (!@oci_execute($result, ($DB['TRANSACTIONS'] ? OCI_DEFAULT : OCI_COMMIT_ON_SUCCESS))) {
 				$e = oci_error($result);
-				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', 'sql');
+				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', true);
 			}
+
 			break;
 	}
 
-	// $result is false only if an error occurred
-	if ($DB['TRANSACTION_NO_FAILED_SQLS'] && !$result) {
+	if (!$result) {
 		$DB['TRANSACTION_NO_FAILED_SQLS'] = false;
 	}
 
@@ -303,7 +317,7 @@ function DBaddLimit($query, $limit = 0, $offset = 0) {
 
 	if ((isset($limit) && ($limit < 0 || !zbx_ctype_digit($limit))) || $offset < 0 || !zbx_ctype_digit($offset)) {
 		$moreDetails = isset($limit) ? ' Limit ['.$limit.'] Offset ['.$offset.']' : ' Offset ['.$offset.']';
-		error('Incorrect parameters for limit and/or offset. Query ['.$query.']'.$moreDetails, 'sql');
+		error('Incorrect parameters for limit and/or offset. Query ['.$query.']'.$moreDetails, true);
 
 		return false;
 	}
@@ -326,44 +340,56 @@ function DBaddLimit($query, $limit = 0, $offset = 0) {
 	return $query;
 }
 
-function DBexecute($query) {
+/**
+ * @param $query
+ *
+ * @return bool
+ */
+function DBexecute($query): bool {
 	global $DB;
 
-	if (!isset($DB['DB']) || empty($DB['DB'])) {
+	if (!array_key_exists('DB', $DB) || $DB['DB'] === null) {
 		return false;
 	}
 
-	$result = false;
 	$time_start = microtime(true);
 
 	$DB['EXECUTE_COUNT']++;
 
+	$result = false;
+
 	switch ($DB['TYPE']) {
 		case ZBX_DB_MYSQL:
-			if (!$result = mysqli_query($DB['DB'], $query)) {
-				error('Error in query ['.$query.'] ['.mysqli_error($DB['DB']).']', 'sql');
+			try {
+				$result = mysqli_query($DB['DB'], $query);
 			}
+			catch (mysqli_sql_exception $e) {
+				trigger_error('Error in query ['.$query.'] ['.$e->getMessage().']', E_USER_WARNING);
+			}
+
 			break;
+
 		case ZBX_DB_POSTGRESQL:
 			if (!$result = (bool) pg_query($DB['DB'], $query)) {
-				error('Error in query ['.$query.'] ['.pg_last_error().']', 'sql');
+				error('Error in query ['.$query.'] ['.pg_last_error($DB['DB']).']', true);
 			}
+
 			break;
+
 		case ZBX_DB_ORACLE:
-			if (!$result = oci_parse($DB['DB'], $query)) {
-				$e = @oci_error();
-				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', 'sql');
-			}
-			elseif (!@oci_execute($result, ($DB['TRANSACTIONS'] ? OCI_DEFAULT : OCI_COMMIT_ON_SUCCESS))) {
-				$e = oci_error($result);
-				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', 'sql');
-			}
-			else {
-				$result = true; // function must return boolean
+			$handle = oci_parse($DB['DB'], $query);
+
+			$result = $handle && @oci_execute($handle, $DB['TRANSACTIONS'] ? OCI_DEFAULT : OCI_COMMIT_ON_SUCCESS);
+
+			if (!$result) {
+				$error = $handle ? oci_error($handle) : oci_error();
+				error('SQL error ['.$error['message'].'] in ['.$error['sqltext'].']', true);
 			}
+
 			break;
 	}
-	if ($DB['TRANSACTIONS'] != 0 && !$result) {
+
+	if (!$result) {
 		$DB['TRANSACTION_NO_FAILED_SQLS'] = false;
 	}
 
@@ -376,49 +402,69 @@ function DBexecute($query) {
 }
 
 /**
- * Returns the next data set from a DB resource or false if there are no more results.
+ * Return the next data set from a DB resource or false if there are no more results.
  *
- * @param resource $cursor
- * @param bool     $convertNulls  Convert all null values to string zeros.
+ * @param mixed $cursor        A DB-specific resource returned by DBselect or DBexecute.
+ * @param bool  $convertNulls  Convert all null values to string zeros.
  *
  * @return array|bool
  */
 function DBfetch($cursor, $convertNulls = true) {
 	global $DB;
 
-	$result = false;
-
-	if (!isset($DB['DB']) || empty($DB['DB']) || is_bool($cursor)) {
-		return $result;
+	if (!array_key_exists('DB', $DB) || $DB['DB'] === null || $cursor === false) {
+		return false;
 	}
 
+	$result = false;
+
 	switch ($DB['TYPE']) {
 		case ZBX_DB_MYSQL:
 			$result = mysqli_fetch_assoc($cursor);
+
 			if (!$result) {
 				mysqli_free_result($cursor);
 			}
+
 			break;
+
 		case ZBX_DB_POSTGRESQL:
-			if (!$result = pg_fetch_assoc($cursor)) {
+			if ($result = pg_fetch_assoc($cursor)) {
+				$i = 0;
+
+				foreach ($result as &$value) {
+					if (pg_field_type($cursor, $i++) === 'bytea') {
+						$value = pg_unescape_bytea($value);
+					}
+				}
+				unset($value);
+			}
+			else {
 				pg_free_result($cursor);
 			}
+
 			break;
+
 		case ZBX_DB_ORACLE:
 			if ($row = oci_fetch_assoc($cursor)) {
 				$result = [];
+
 				foreach ($row as $key => $value) {
 					$field_type = strtolower(oci_field_type($cursor, $key));
-					// Oracle does not support NULL values for string fields, so if the string is empty, it will return NULL
-					// convert it to an empty string to be consistent with other databases
-					$value = (str_in_array($field_type, ['varchar', 'varchar2', 'blob', 'clob']) && is_null($value)) ? '' : $value;
+
+					// Since Oracle reports nulls for empty strings, convert those back to empty strings.
+					$value = (str_in_array($field_type, ['varchar', 'varchar2', 'blob', 'clob']) && is_null($value))
+						? ''
+						: $value;
 
 					if (is_object($value) && (strpos($field_type, 'lob') !== false)) {
 						$value = $value->load();
 					}
+
 					$result[strtolower($key)] = $value;
 				}
 			}
+
 			break;
 	}
 
@@ -499,22 +545,13 @@ function get_dbid($table, $field) {
 	return $ret2;
 }
 
-function zbx_db_distinct($sql_parts) {
-	$count = count($sql_parts['from']);
-	if (array_key_exists('left_join', $sql_parts)) {
-		$count += count($sql_parts['left_join']);
-	}
-
-	return ($count > 1 ? ' DISTINCT' : '');
-}
-
 function zbx_db_search($table, $options, &$sql_parts) {
 	global $DB;
 	list($table, $tableShort) = explode(' ', $table);
 
 	$tableSchema = DB::getSchema($table);
 	if (!$tableSchema) {
-		info(_s('Error in search request for table "%1$s".', $table));
+		error(_s('Error in search request for table "%1$s".', $table));
 	}
 
 	$start = $options['startSearch'] ? '' : '%';
@@ -537,7 +574,8 @@ function zbx_db_search($table, $options, &$sql_parts) {
 
 		if ($tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_CHAR
 				&& $tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_NCLOB
-				&& $tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_TEXT) {
+				&& $tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_TEXT
+				&& $tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_CUID) {
 			continue;
 		}
 
@@ -559,7 +597,7 @@ function zbx_db_search($table, $options, &$sql_parts) {
 				$pattern = zbx_dbstr($pattern);
 			}
 
-			$fieldSearch[] = 'UPPER('.$tableShort.'.'.$field.')'.$exclude.' LIKE '.$pattern." ESCAPE '!'";
+			$fieldSearch[] = DB::uppercaseField($field, $table, $tableShort).$exclude.' LIKE '.$pattern." ESCAPE '!'";
 		}
 
 		$search[$field] = '('.implode($glue, $fieldSearch).')';
@@ -607,17 +645,12 @@ function check_db_fields($dbFields, &$args) {
 }
 
 /**
- * Takes an initial part of SQL query and appends a generated WHERE condition.
- * The WHERE condition is generated from the given list of values as a mix of
- * <fieldname> BETWEEN <id1> AND <idN>" and "<fieldname> IN (<id1>,<id2>,...,<idN>)" elements.
+ * Create condition SQL for field matching against numeric values.
  *
- * In some frontend places we can get array with bool as input values parameter. This is fail!
- * Therefore we need check it and return 1=0 as temporary solution to not break the frontend.
- *
- * @param string $field_name    Field name to be used in SQL WHERE condition
- * @param array  $values        Array of numerical values sorted in ascending order to be included in WHERE
- * @param bool   $not_in        Builds inverted condition
- * @param bool   $zero_to_null  Cast zero to null
+ * @param string $field_name
+ * @param array  $values
+ * @param bool   $not_in        Create inverse condition.
+ * @param bool   $zero_to_null  Cast zero to null.
  *
  * @return string
  */
@@ -628,7 +661,7 @@ function dbConditionInt($field_name, array $values, $not_in = false, $zero_to_nu
 	$MAX_NUM_IN = 950; // Maximum number of values for using "IN (<id1>,<id2>,...,<idN>)".
 
 	if (is_bool(reset($values))) {
-		return '1=0';
+		return $not_in ? '1=1' : '1=0';
 	}
 
 	$values = array_flip($values);
@@ -909,11 +942,11 @@ function zbx_dbstr($var) {
 		case ZBX_DB_POSTGRESQL:
 			if (is_array($var)) {
 				foreach ($var as $vnum => $value) {
-					$var[$vnum] = "'".pg_escape_string($value)."'";
+					$var[$vnum] = "'".pg_escape_string($DB['DB'], $value)."'";
 				}
 				return $var;
 			}
-			return "'".pg_escape_string($var)."'";
+			return "'".pg_escape_string($DB['DB'], $var)."'";
 
 		default:
 			return false;
